/***************************************************************************
 *                                                                         *
 *   KCPULoad and KNetLoad are copyright (c) 1999-2000, Markus Gustavsson  *
 *                                       (c) 2002, Ben Burton              *
 *                                       (c) 2004, Diego Petten           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "statdock.h"
#include "statpopup.h"
#include "settings.h"
#include "visualization.h"

#include <qpainter.h>
#include <qtimer.h>
#include <qcheckbox.h>
#include <kaction.h>
#include <kcolordialog.h>
#include <kconfigdialog.h>
#include <kdebug.h>
#include <kglobal.h>
#include <khelpmenu.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kpopupmenu.h>
#include <kwin.h>

#define TEXT_EXPANSION_HORIZONTAL 10
#define TEXT_EXPANSION_VERTICAL 3

const QColor StatPopup::colorBorder(0, 0, 0);

StatPopup::StatPopup(QWidget *parent, const char *name) :
        QWidget(parent, name, Qt::WStyle_Customize | Qt::WStyle_NoBorder |
        Qt::WStyle_StaysOnTop | Qt::WDestructiveClose | Qt::WType_TopLevel)
        {
    // Window management.
    KWin::setState(winId(), NET::SkipTaskbar | NET::SkipPager);

    // TODO: Get all-desktops working, even after a hide/reshow.
    //KWin::setOnAllDesktops(winId(), true);

    // Basic variable initialisation.
    relX = relY = 0;
    dock[0] = dock[1] = 0;
    isDragged = closing = false;

    // Initialise the text contents of this pop-up.
    fullReading = i18n("Inactive.");
    resizeToText();

    // Prepare for actions, but don't actually add any actions.
    coll = new KActionCollection(this);

    // Set up a timer for our periodic updates.
    timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(takeReading()));

    VisualizationConfig *viscfg = new VisualizationConfig(0, "Visualization");
#if STATPOPUP_SPLITTABLE == 0
    delete viscfg->kcfg_Split;
#endif
    
    cfgdialog = new KConfigDialog(this, "settings", Settings::self());
    cfgdialog->addPage(viscfg, i18n("Visualization"), "style");
    connect(cfgdialog, SIGNAL(settingsChanged()), this, SLOT(reloadSettings()));
}

StatPopup::~StatPopup() {
}

void StatPopup::readPopupState() {
	// Read the position.
	if ( Settings::popupX() || Settings::popupY() )
		move(Settings::popupX(), Settings::popupY());
	
	// Show the pop-up if appropriate.
	if ( Settings::popupVisible() )
		show();
	else
		hide();
}

void StatPopup::savePopupState() {
	Settings::setPopupX(x());
	Settings::setPopupY(y());
	Settings::setPopupVisible(isVisible());
	Settings::self()->writeConfig();
}

void StatPopup::initDock(StatDock* target, KPopupMenu* menu) {
    // Initialise the menus.
    actClearHistory->plug(menu);
    actSettings->plug(menu);
#if STATPOPUP_SPLITTABLE == 1
    actSplit->plug(menu);
#endif
    menu->insertSeparator();

    insertCustomItems(menu);

    menu->insertItem(SmallIcon("help"), i18n("&Help"),
        (new KHelpMenu(0, KGlobal::instance()->aboutData(), false))->menu());

    reloadDock(target);
    reloadSettings();
}

void StatPopup::clearHistory() {
    if (dock[0])
        dock[0]->clearHistory();
    if (dock[1])
        dock[1]->clearHistory();
}

StatDock* StatPopup::firstDock() {
    if (dock[0])
        return dock[0];
    else
        return dock[1];
}

void StatPopup::startUpdates() {
    takeReading();
    timer->start(Settings::timeout());
}

void StatPopup::stopUpdates() {
    timer->stop();
    fullReading = i18n("Inactive.");
}

void StatPopup::setupActions() {

    actClearHistory = new KAction(i18n("&Clear"), "editdelete", 0,
        this, SLOT(clearHistory()), coll, "clear");

    actSettings = new KAction(i18n("&Configure KNetLoad..."), "configure", 0,
        this, SLOT(showSettings()), coll, "settings");

    setupCustomActions();
}

void StatPopup::paintEvent(QPaintEvent*) {
    QPainter p(this);

    // Draw the border.
    p.setPen(colorBorder);
    p.drawLine(0, 0, width(), 0);
    p.drawLine(0, 1, width(), 1);
    p.drawLine(0, 2, 0, height());
    p.drawLine(1, 2, 1, height());
    p.drawLine(width() - 2, 2, width() - 2, height());
    p.drawLine(width() - 1, 2, width() - 1, height());
    p.drawLine(2, height() - 2, width() - 2, height() - 2);
    p.drawLine(2, height() - 1, width() - 2, height() - 1);

    // Draw the text.
    p.setFont(font());
    p.setPen(colorGroup().foreground());
    p.drawText(rect(), AlignHCenter | AlignVCenter, fullReading);
}

void StatPopup::mousePressEvent(QMouseEvent* e) {
    if(e->button() == RightButton) {
        // Hide the pop-up.
        hide();
    } else {
        // Begin a drag operation.
        isDragged = true;
        relX = e->x();
        relY = e->y();
        repaint();
    }
}

void StatPopup::mouseMoveEvent(QMouseEvent* e) {
    // In the middle of a drag operation.
    move(e->globalX() - relX, e->globalY() - relY);
}

void StatPopup::mouseReleaseEvent(QMouseEvent* e) {
    // The end of a drag operation.
    move(e->globalX() - relX, e->globalY() - relY);
    isDragged = false;
    repaint();
}

void StatPopup::closeEvent(QCloseEvent* e) {
    // We're about to close.  Save the current state for the last time.
    savePopupState();
    closing = true;
    QWidget::closeEvent(e);
}

void StatPopup::hideEvent(QHideEvent* e) {
    // We're about to hide.  Save the current state if we're not
    // closing altogether.
    if (! closing)
        savePopupState();
    QWidget::hideEvent(e);
}

void StatPopup::showEvent(QShowEvent* e) {
    closing = false;
    // Make sure we're up-to-date and properly resized.
    if (Settings::enabled())
        takeReading();
    else
        resizeToText();

    QWidget::showEvent(e);
}

void StatPopup::takeReading() {
    takeReadingInternal();

    if (dock[0])
        dock[0]->addPercentReading(upper[0], lower[0]);
    if (dock[1])
        dock[1]->addPercentReading(upper[1], lower[1]);

    if (isVisible()) {
        // Only resize if the pop-up is visible; otherwise fullReading
        // might be stale anyway so we'll do it when we show the pop-up.
        if (resizeRequested)
            resizeToText();

        repaint();
    }
}

void StatPopup::resizeToText() {
    resizeRequested = false;

    QSize size = fontMetrics().size(0, fullReading);
    resize(size.width() + 2 * TEXT_EXPANSION_HORIZONTAL,
        size.height() + 2 * TEXT_EXPANSION_VERTICAL);
    repaint();
}

void StatPopup::showSettings() {
	if(KConfigDialog::showDialog("settings"))
		return;
	
	cfgdialog->show();
}

void StatPopup::reloadSettings() {
	// Set up display properties for the dock.
	if (Settings::enabled())
		startUpdates();
	else
		stopUpdates();
	
	if (dock[0])
		reloadDock(dock[0]);
	if (dock[1])
		reloadDock(dock[1]);
}

void StatPopup::reloadDock(StatDock *dck)
{
	setCustomProperties(dck);
	dck->repaint();
}

#include "statpopup.moc"
